@@ -2,7 +2,6 @@ import torch
 import torch.nn as nn
 import torch.nn.functional as F
 import torch.linalg as LA
-from torch.amp import autocast
 from torch.nn.attention.flex_attention import flex_attention, create_block_mask
 import math
 import warnings
@@ -50,7 +49,7 @@ class MultiheadAttention(nn.Module):
         if self.out_proj.bias is not None:
             nn.init.constant_(self.out_proj.bias, 0.)
         
-    def forward(self, x, attn_mask=None, rope=None):
+    def forward(self, x, causal=False, rope=None):
         # Handle batch_first option
         if self.batch_first:
             x = x.transpose(0, 1)
@@ -72,45 +71,32 @@ class MultiheadAttention(nn.Module):
                 attn_mask = F.pad(attn_mask, (0, 1))
         
         # Reshape q, k, v for multi-head attention
-        q = q.contiguous().view(tgt_len, bsz * self.n_heads, self.d_head).transpose(0, 1)
-        k = k.contiguous().view(src_len, bsz * self.n_heads, self.d_head).transpose(0, 1)
-        v = v.contiguous().view(src_len, bsz * self.n_heads, self.d_head).transpose(0, 1)
+        q = rearrange(q, 's b (h d) -> b h s d', h=self.n_heads).contiguous()  # (bsz, n_heads, tgt_len, d_head)
+        k = rearrange(k, 's b (h d) -> b h s d', h=self.n_heads).contiguous()  # (bsz, n_heads, src_len, d_head)
+        v = rearrange(v, 's b (h d) -> b h s d', h=self.n_heads).contiguous()  # (bsz, n_heads, src_len, d_head)
         
         if rope:
             if rope.use_xpos:
-                q, k = rope.rotate_queries_and_keys(q.reshape(bsz, self.n_heads, tgt_len, self.d_head), k.reshape(bsz, self.n_heads, src_len, self.d_head))
+                q, k = rope.rotate_queries_and_keys(q, k)
             else:
                 q = rope.rotate_queries_or_keys(q)
                 k = rope.rotate_queries_or_keys(k)
-        q = q.reshape(bsz * self.n_heads, tgt_len, self.d_head).contiguous()
-        k = k.reshape(bsz * self.n_heads, src_len, self.d_head).contiguous()
         
         # Calculate attention scores
-        beta = torch.exp(self.beta).reshape(self.n_heads, 1, 1).repeat(bsz, 1, 1)
-        # beta = F.softplus(self.beta).reshape(self.n_heads, 1, 1).repeat(bsz, 1, 1)
+        beta = torch.exp(self.beta).reshape(1, self.n_heads, 1, 1)
+        # beta = F.softplus(self.beta).reshape(1, self.n_heads, 1, 1)
         q = q / (math.sqrt(self.d_head) * beta)
-        attn_output_weights = torch.bmm(q, k.transpose(1, 2))  # (bsz * n_heads, tgt_len, src_len)
         
-        # Apply attention mask if provided
-        if attn_mask is not None:
-            if attn_mask.dim() == 2:
-                attn_mask = attn_mask.unsqueeze(0)
-            elif attn_mask.dim() == 3:
-                attn_mask = attn_mask.repeat(self.n_heads, 1, 1)
-            attn_output_weights = attn_output_weights + attn_mask
+        q = q.flatten(0, 1)
+        k = k.flatten(0, 1)
+        v = v.flatten(0, 1)
         
-        # Convert attention weights to probabilities
-        attn_output_weights = F.softmax(attn_output_weights, dim=-1)
-        attn_output_weights = F.dropout(attn_output_weights, p=self.dropout, training=self.training)
-        
-        # Apply attention weights to values
-        attn_output = torch.bmm(attn_output_weights, v)  # (bsz * n_heads, tgt_len, d_head)
+        attn_output = F.scaled_dot_product_attention(q, k, v, scale=1.0, dropout_p=self.dropout, is_causal=causal)  # (bsz * n_heads, src_len, d_head)
         
         # Reshape output
-        attn_output = attn_output.transpose(0, 1).contiguous().view(tgt_len, bsz, d_model)
+        attn_output = rearrange(attn_output, '(b h) s d -> s b (h d)', h=self.n_heads).contiguous()  # (tgt_len, bsz, d_model)
         attn_output = self.out_proj(attn_output)
         
-        # Return in the correct format depending on batch_first
         if self.batch_first:
             return attn_output.transpose(0, 1)
         return attn_output
@@ -120,12 +106,13 @@ class LinearAttention(nn.Module):
     Vanilla Linear Attention.
     Kernel function is softplus.
     """
-    def __init__(self, d_model, n_heads, bias=True, attn_sink=False, device="cpu"):
+    def __init__(self, d_model, n_heads, bias=True, attn_sink=False, batch_first=False, device="cpu"):
         super().__init__()
         self.d_model = d_model
         self.n_heads = n_heads
         self.d_head = d_model // n_heads
         self.attn_sink = attn_sink
+        self.batch_first = batch_first
         self.device = device
         
         self.beta = nn.Parameter(torch.empty(self.n_heads, device=device))
@@ -154,11 +141,14 @@ class LinearAttention(nn.Module):
             nn.init.constant_(self.out_proj.bias, 0.)
     
     def forward(self, x, rope=None, causal=True):
-        bsz, src_len, d_model = x.size()
+        if self.batch_first:
+            x = x.transpose(0, 1)
+        
+        src_len, bsz, d_model = x.size()
         tgt_len = src_len
-        q = rearrange(self.q_proj(x), 'b s (h d) -> b h s d', h=self.n_heads)
-        k = rearrange(self.k_proj(x), 'b s (h d) -> b h s d', h=self.n_heads)
-        v = rearrange(self.v_proj(x), 'b s (h d) -> (b h) s d', h=self.n_heads).contiguous()
+        q = rearrange(self.q_proj(x), 's b (h d) -> b h s d', h=self.n_heads)
+        k = rearrange(self.k_proj(x), 's b (h d) -> b h s d', h=self.n_heads)
+        v = rearrange(self.v_proj(x), 's b (h d) -> (b h) s d', h=self.n_heads).contiguous()
         
         if self.attn_sink:
             src_len += 1
@@ -171,14 +161,15 @@ class LinearAttention(nn.Module):
             else:
                 q = rope.rotate_queries_or_keys(q)
                 k = rope.rotate_queries_or_keys(k)
-        q = q.reshape(bsz * self.n_heads, tgt_len, self.d_head).contiguous()
-        k = k.reshape(bsz * self.n_heads, src_len, self.d_head).contiguous()
         
-        beta = torch.exp(self.beta).reshape(self.n_heads, 1, 1).repeat(bsz, 1, 1)
-        # beta = F.softplus(self.beta).reshape(self.n_heads, 1, 1).repeat(bsz, 1, 1)
+        beta = torch.exp(self.beta).reshape(1, self.n_heads, 1, 1)
+        # beta = F.softplus(self.beta).reshape(1, self.n_heads, 1, 1)
         q = q / (math.sqrt(self.d_head) * beta)
         k = k / (math.sqrt(self.d_head) * beta)
         
+        q = q.flatten(0, 1).contiguous()
+        k = k.flatten(0, 1).contiguous()
+        
         # q = torch.exp(q)
         # k = torch.exp(k)
         # q = F.elu(q) + 1
@@ -195,19 +186,24 @@ class LinearAttention(nn.Module):
         
         out = torch.matmul(q.unsqueeze(-2), kv).squeeze(-2) / torch.matmul(q.unsqueeze(-2), kn.unsqueeze(-1)).squeeze(-1)
         out = rearrange(out, '(b h) s d -> b s (h d)', h=self.n_heads)
-        return self.out_proj(out)
+        out = self.out_proj(out)
+        
+        if self.batch_first:
+            out = out.transpose(0, 1)
+        return out
 
 class OrthoLinearAttention(nn.Module):
     """
     Orthogonal Linear Attention.
     A derivative of linear attention that orthogonalizes queries and keys for each head to reduce crossterm interference.
     """
-    def __init__(self, d_model, n_heads, bias=True, attn_sink=False, device="cpu"):
+    def __init__(self, d_model, n_heads, bias=True, attn_sink=False, batch_first=False, device="cpu"):
         super().__init__()
         self.d_model = d_model
         self.n_heads = n_heads
         self.d_head = d_model // n_heads
         self.attn_sink = attn_sink
+        self.batch_first = batch_first
         self.device = device
         
         self.beta = nn.Parameter(torch.empty(self.n_heads, device=device))
@@ -236,11 +232,14 @@ class OrthoLinearAttention(nn.Module):
             nn.init.constant_(self.out_proj.bias, 0.)
     
     def forward(self, x, rope=None, causal=True):
-        bsz, src_len, d_model = x.size()
+        if self.batch_first:
+            x = x.transpose(0, 1)
+        
+        src_len, bsz, d_model = x.size()
         tgt_len = src_len
-        q = rearrange(self.q_proj(x), 'b s (h d) -> b h s d', h=self.n_heads)
-        k = rearrange(self.k_proj(x), 'b s (h d) -> b h s d', h=self.n_heads)
-        v = rearrange(self.v_proj(x), 'b s (h d) -> (b h) s d', h=self.n_heads).contiguous()
+        q = rearrange(self.q_proj(x), 's b (h d) -> b h s d', h=self.n_heads)
+        k = rearrange(self.k_proj(x), 's b (h d) -> b h s d', h=self.n_heads)
+        v = rearrange(self.v_proj(x), 's b (h d) -> (b h) s d', h=self.n_heads).contiguous()
         
         if self.attn_sink:
             src_len += 1
@@ -253,14 +252,15 @@ class OrthoLinearAttention(nn.Module):
             else:
                 q = rope.rotate_queries_or_keys(q)
                 k = rope.rotate_queries_or_keys(k)
-        q = q.reshape(bsz * self.n_heads, tgt_len, self.d_head).contiguous()
-        k = k.reshape(bsz * self.n_heads, src_len, self.d_head).contiguous()
-        
-        beta = torch.exp(self.beta).reshape(self.n_heads, 1, 1).repeat(bsz, 1, 1)
-        # beta = F.softplus(self.beta).reshape(self.n_heads, 1, 1).repeat(bsz, 1, 1)
+                
+        beta = torch.exp(self.beta).reshape(1, self.n_heads, 1, 1)
+        # beta = F.softplus(self.beta).reshape(1, self.n_heads, 1, 1)
         q = q * beta
         k = k * beta
         
+        q = q.flatten(0, 1).contiguous()
+        k = k.flatten(0, 1).contiguous()
+        
         q = q.softmax(-1)
         k = k.softmax(-1)
         
@@ -273,7 +273,11 @@ class OrthoLinearAttention(nn.Module):
         
         out = torch.matmul(q.unsqueeze(-2), kv).squeeze(-2) / torch.matmul(q.unsqueeze(-2), kn.unsqueeze(-1)).squeeze(-1)
         out = rearrange(out, '(b h) s d -> b s (h d)', h=self.n_heads)
-        return self.out_proj(out)
+        out = self.out_proj(out)
+        
+        if self.batch_first:
+            out = out.transpose(0, 1)
+        return out
 
 class CompressionAttention(nn.Module):
     """
@@ -304,6 +308,9 @@ class CompressionAttention(nn.Module):
 
         self._reset_parameters()
         
+        if device == "mps":
+            warnings.warn("Kernelized attention with q_dim != v_dim is broken on MPS. Falling back to PyTorch implementation", UserWarning)
+        
     def _reset_parameters(self):
         # Initialize projections using Xavier uniform
         nn.init.constant_(self.beta, 0.)
@@ -325,7 +332,7 @@ class CompressionAttention(nn.Module):
     def forward(self, x, rope=None, causal=True):
         if self.batch_first:
             x = x.transpose(0, 1)
-        
+                
         cmprs_len = self.compressed_len
         src_len, bsz, d_model = x.shape
         tgt_len = src_len
@@ -341,72 +348,57 @@ class CompressionAttention(nn.Module):
             v_s = torch.cat([v_s, torch.zeros((1, bsz, d_model), dtype=x.dtype, device=x.device)], dim=0)
         
         # Reshape for multi-head attention
-        q_c = rearrange(q_c, 'c b (h d) -> (b h) c d', h=self.n_heads).contiguous()
-        q_s = rearrange(q_s, 's b (h d) -> (b h) s d', h=self.n_heads).contiguous()
-        k_s = rearrange(k_s, 's b (h d) -> (b h) s d', h=self.n_heads).contiguous()
-        v_s = rearrange(v_s, 's b (h d) -> (b h) s d', h=self.n_heads).contiguous()
-        
+        q_c = rearrange(q_c, 'c b (h d) -> b h c d', h=self.n_heads).contiguous()
+        q_s = rearrange(q_s, 's b (h d) -> b h s d', h=self.n_heads).contiguous()
+        k_s = rearrange(k_s, 's b (h d) -> b h s d', h=self.n_heads).contiguous()
+        v_s = rearrange(v_s, 's b (h d) -> b h s d', h=self.n_heads).contiguous()
+                
         if rope:
             if rope.use_xpos:
-                q_s, k_s = rope.rotate_queries_and_keys(q_s.reshape(bsz, self.n_heads, tgt_len, self.d_head), k_s.reshape(bsz, self.n_heads, src_len, self.d_head))
+                q_s, k_s = rope.rotate_queries_and_keys(q_s, k_s)
             else:
-                q_s = rope.rotate_queries_or_keys(q_s.reshape(bsz, self.n_heads, tgt_len, self.d_head))
-                k_s = rope.rotate_queries_or_keys(k_s.reshape(bsz, self.n_heads, src_len, self.d_head))
-            q_s = q_s.reshape(bsz * self.n_heads, tgt_len, self.d_head).contiguous()
-            k_s = k_s.reshape(bsz * self.n_heads, src_len, self.d_head).contiguous()
+                q_s = rope.rotate_queries_or_keys(q_s)
+                k_s = rope.rotate_queries_or_keys(k_s)
+        
+        beta = torch.exp(self.beta).reshape(1, self.n_heads, 1, 1)
+        # beta = F.softplus(self.beta).reshape(1, self.n_heads, 1, 1)
+        q_s = q_s / (math.sqrt(self.d_head) * beta)
+        
+        q_c = q_c.flatten(0, 1)
+        q_s = q_s.flatten(0, 1)
+        k_s = k_s.flatten(0, 1)
+        v_s = v_s.flatten(0, 1)
         
         kv_s = torch.cat([k_s, v_s], dim=-1)  # (bsz * n_heads, src_len, 2*d_head)
         
-        ### Compression self attention ###
-        c_attn_weights = torch.bmm(q_c, k_s.transpose(1, 2))  # (bsz * n_heads, cmprs_len, src_len)
-        
         if causal:
             # Manually perform softmax with cumulative sum for causal attention
+            c_attn_weights = torch.matmul(q_c, k_s.transpose(-2, -1))  # (bsz * n_heads, cmprs_len, src_len)
             c_attn_weights = torch.exp(c_attn_weights - torch.max(c_attn_weights, dim=-1, keepdim=True).values)  # (bsz * n_heads, cmprs_len, src_len)
-            c_attn_norm = torch.cumsum(c_attn_weights, dim=-1)  # (bsz * n_heads, cmprs_len, src_len)
             c_attn_weights = F.dropout(c_attn_weights, p=self.dropout, training=self.training)
-        else:
-            # Convert attention weights to probabilities
-            c_attn_weights = F.softmax(c_attn_weights, dim=-1)
-            c_attn_weights = F.dropout(c_attn_weights, p=self.dropout, training=self.training)
-        
-        ### Expansion self attention ###
-        beta = torch.exp(self.beta).reshape(self.n_heads, 1, 1).repeat(bsz, 1, 1)
-        # beta = F.softplus(self.beta).reshape(self.n_heads, 1, 1).repeat(bsz, 1, 1)
-        q_s = q_s / (math.sqrt(self.d_head) * beta)
-        
-        if causal:
+            
             # Calculate attention scores for compressed output
-            kv_c = torch.cumsum((c_attn_weights.unsqueeze(-1) * kv_s.unsqueeze(1)), dim=2) / c_attn_norm.unsqueeze(-1)  # (bsz * n_heads, cmprs_len, src_len, 2*d_head)
-            k_c, v_c = kv_c.split([self.d_head, self.d_head], dim=-1)  # (bsz * n_heads, cmprs_len, src_len, d_head)
-            s_attn_weights = torch.einsum('zsd, zcsd -> zsc', q_s, k_c)  # (bsz * n_heads, tgt_len, cmprs_len)
+            kv_c = torch.cumsum((c_attn_weights.unsqueeze(-1) * kv_s.unsqueeze(1)), dim=-2) / torch.cumsum(c_attn_weights, dim=-1).unsqueeze(-1)  # (bsz * n_heads, cmprs_len, src_len, 2*d_head)
             
-            # Convert attention weights to probabilities
-            s_attn_weights = F.softmax(s_attn_weights, dim=-1)
-            s_attn_weights = F.dropout(s_attn_weights, p=self.dropout, training=self.training)
-            
-            # Apply attention weights to values
-            s_attn_output = torch.einsum('zsc, zcsd -> zsd', s_attn_weights, v_c)  # (bsz * n_heads, tgt_len, d_head)
+            k_c, v_c = kv_c.transpose(1, 2).split([self.d_head, self.d_head], dim=-1)  # (bsz, n_heads, cmprs_len, src_len, d_head)
+            s_attn_output = F.scaled_dot_product_attention(q_s.unsqueeze(-2), k_c, v_c, scale=1.0, dropout_p=self.dropout, is_causal=False).squeeze(-2)  # (bsz * n_heads, tgt_len, d_head)
         else:
             # Calculate attention scores for compressed output
-            kv_c = torch.bmm(c_attn_weights, kv_s)  # (bsz * n_heads, cmprs_len, 2*d_head)
+            if self.device == "mps":
+                c_attn_weights = torch.matmul(q_c, k_s.transpose(-2, -1))  # (bsz * n_heads, cmprs_len, src_len)
+                c_attn_weights = F.softmax(c_attn_weights, dim=-1)
+                kv_c = torch.matmul(c_attn_weights, kv_s)  # (bsz * n_heads, cmprs_len, src_len)
+            else:
+                kv_c = F.scaled_dot_product_attention(q_c, k_s, kv_s, scale=1.0, dropout_p=self.dropout, is_causal=False)  # (bsz * n_heads, cmprs_len, 2*d_head)
             k_c, v_c = kv_c.split([self.d_head, self.d_head], dim=-1)  # (bsz * n_heads, cmprs_len, d_head)
-            s_attn_weights = torch.bmm(q_s, k_c.transpose(1, 2))  # (bsz * n_heads, tgt_len, cmprs_len)
-            
-            # Convert attention weights to probabilities
-            s_attn_weights = F.softmax(s_attn_weights, dim=-1)
-            s_attn_weights = F.dropout(s_attn_weights, p=self.dropout, training=self.training)
-            
-            # Apply attention weights to values
-            s_attn_output = torch.bmm(s_attn_weights, v_c)  # (bsz * n_heads, tgt_len, d_head)
+            s_attn_output = F.scaled_dot_product_attention(q_s, k_c, v_c, scale=1.0, dropout_p=self.dropout, is_causal=False)  # (bsz * n_heads, tgt_len, d_head)
         
         # Reshape output
-        s_attn_output = s_attn_output.transpose(0, 1).contiguous().view(tgt_len, bsz, d_model)
+        s_attn_output = rearrange(s_attn_output, '(b h) s d -> s b (h d)', h=self.n_heads).contiguous()  # (tgt_len, bsz, d_model)
         
         # Apply final projection
         s_attn_output = self.out_proj(s_attn_output)
         
-        # Return in the correct format depending on batch_first
         if self.batch_first:
             return s_attn_output.transpose(0, 1)
         return s_attn_output
@@ -528,82 +520,67 @@ class SlidingWindowAttention(nn.Module):
         v = self.v_proj(x)  # (src_len, batch_size, d_model)
         
         # Reshape for multi-head attention
-        q = rearrange(q, 's b (h d) -> (b h) s d', h=self.n_heads).contiguous()
-        k = rearrange(k, 's b (h d) -> (b h) s d', h=self.n_heads).contiguous()
-        v = rearrange(v, 's b (h d) -> (b h) s d', h=self.n_heads).contiguous()
+        q = rearrange(q, 's b (h d) -> b h s d', h=self.n_heads).contiguous()  # (bsz, n_heads, tgt_len, d_head)
+        k = rearrange(k, 's b (h d) -> b h s d', h=self.n_heads).contiguous()  # (bsz, n_heads, src_len, d_head)
+        v = rearrange(v, 's b (h d) -> b h s d', h=self.n_heads).contiguous()  # (bsz, n_heads, src_len, d_head)
         
         if rope:
             if rope.use_xpos:
-                q, k = rope.rotate_queries_and_keys(q.reshape(bsz, self.n_heads, src_len, self.d_head), k.reshape(bsz, self.n_heads, src_len, self.d_head))
+                q, k = rope.rotate_queries_and_keys(q, k)
             else:
-                q = rope.rotate_queries_or_keys(q.reshape(bsz, self.n_heads, src_len, self.d_head))
-                k = rope.rotate_queries_or_keys(k.reshape(bsz, self.n_heads, src_len, self.d_head))
-            q = q.reshape(bsz * self.n_heads, src_len, self.d_head).contiguous()
-            k = k.reshape(bsz * self.n_heads, src_len, self.d_head).contiguous()
+                q = rope.rotate_queries_or_keys(q)
+                k = rope.rotate_queries_or_keys(k)
         
-        beta = torch.exp(self.beta).reshape(self.n_heads, 1, 1).repeat(bsz, 1, 1)
-        # beta = F.softplus(self.beta).reshape(self.n_heads, 1, 1).repeat(bsz, 1, 1)
-        q = q / (math.sqrt(self.d_head) * beta).to(q.dtype)
+        beta = torch.exp(self.beta).reshape(1, self.n_heads, 1, 1)
+        # beta = F.softplus(self.beta).reshape(1, self.n_heads, 1, 1)
+        q = q / (math.sqrt(self.d_head) * beta)
         
         if not self.use_flex_attn:
+            q = q.flatten(0, 1)
+            k = k.flatten(0, 1)
+            v = v.flatten(0, 1)
+            
             if causal:
-                attn_mask = self.causal_windowed_mask(src_len, self.window_len, dilation=self.dilation, to_bias=True)  # (1, src_len, src_len)
+                attn_mask = self.causal_windowed_mask(src_len, self.window_len, dilation=self.dilation, to_bias=True)  # (1, tgt_len, src_len)
             else:
-                attn_mask = self.symmetric_windowed_mask(src_len, self.window_len, dilation=self.dilation, to_bias=True)  # (1, src_len, src_len)
+                attn_mask = self.symmetric_windowed_mask(src_len, self.window_len, dilation=self.dilation, to_bias=True)  # (1, tgt_len, src_len)
             
             if self.attn_sink:
+                src_len += 1
                 k = torch.cat([k, torch.zeros((bsz*self.n_heads, 1, self.d_head), dtype=k.dtype, device=k.device)], dim=1)
                 v = torch.cat([v, torch.zeros((bsz*self.n_heads, 1, self.d_head), dtype=v.dtype, device=v.device)], dim=1)
                 if attn_mask is not None:
-                    attn_mask = F.pad(attn_mask, (0, 1))  # (1, src_len, src_len + 1)
+                    attn_mask = F.pad(attn_mask, (0, 1))  # (1, tgt_len, src_len + 1)
 
-            attn_output_weights = torch.bmm(q, k.transpose(1, 2))  # (bsz * n_heads, src_len, src_len)
-            attn_output_weights = attn_output_weights + attn_mask  # (bsz * n_heads, src_len, src_len)
-            if self.attn_sink:
-                sink_weight = torch.zeros((bsz * self.n_heads, src_len, 1), dtype=x.dtype, device=self.device)  # (bsz * n_heads, src_len, 1)
-                attn_output_weights = torch.cat([attn_output_weights, sink_weight], dim=-1)  # (bsz * n_heads, src_len, src_len + 1)
-            
-            # Convert attention weights to probabilities
-            attn_output_weights = F.softmax(attn_output_weights, dim=-1)
-            attn_output_weights = F.dropout(attn_output_weights, p=self.dropout, training=self.training)
-            
-            # Apply attention weights to values
-            attn_output = torch.bmm(attn_output_weights, v)[:, :src_len]  # (bsz * n_heads, src_len, d_head)
+            attn_output = F.scaled_dot_product_attention(q, k, v, attn_mask=attn_mask, scale=1.0, dropout_p=self.dropout)  # (bsz * n_heads, tgt_len, d_head)
             
             attn_output = rearrange(attn_output, '(b h) s d -> s b (h d)', h=self.n_heads)
         else:
             if self.attn_sink:
+                src_len += 1
                 k = torch.cat([k, torch.zeros((bsz*self.n_heads, 1, self.d_head), dtype=k.dtype, device=k.device)], dim=1)
                 v = torch.cat([v, torch.zeros((bsz*self.n_heads, 1, self.d_head), dtype=v.dtype, device=v.device)], dim=1)
-                tgt_len += 1
             
             if causal:
                 block_mask = self.causal_windowed_block_mask(src_len, tgt_len)
             else:
                 block_mask = self.symmetric_windowed_block_mask(src_len, tgt_len)
             
-            q = rearrange(q, '(b h) s d -> b h s d', h=self.n_heads).contiguous()
-            k = rearrange(k, '(b h) s d -> b h s d', h=self.n_heads).contiguous()
-            v = rearrange(v, '(b h) s d -> b h s d', h=self.n_heads).contiguous()
-            
             attn_device = "cuda" if self.device == "cuda" else "cpu"
             
-            # autocast_dtype = torch.get_autocast_dtype(self.device)
-            
-            q = q.to(attn_device)#.to(autocast_dtype)
-            k = k.to(attn_device)#.to(autocast_dtype)
-            v = v.to(attn_device)#.to(autocast_dtype)
+            q = q.to(attn_device)
+            k = k.to(attn_device)
+            v = v.to(attn_device)
 
-            block_mask = block_mask.to(attn_device)#.to(autocast_dtype)
+            block_mask = block_mask.to(attn_device)
 
-            attn_output = flex_attention(q, k, v, block_mask=block_mask, scale=1.0)[:, :, :src_len]
+            attn_output = flex_attention(q, k, v, block_mask=block_mask, scale=1.0)
             
-            attn_output = rearrange(attn_output, 'b h s d -> s b (h d)', h=self.n_heads).contiguous().to(self.device)
+            attn_output = rearrange(attn_output, '(b h) s d -> s b (h d)', h=self.n_heads).contiguous().to(self.device)
         
         # Apply final projection
         attn_output = self.out_proj(attn_output)
         
-        # Return in the correct format depending on batch_first
         if self.batch_first:
             return attn_output.transpose(0, 1)
         return attn_output
